"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const case_1 = __importDefault(require("case"));
const utils_1 = require("../../utils");
const name = 'object-prop-casing';
const rule = {
    name,
    visitor: {
        SchemaObject: ({ node, report, setting, location }) => {
            if (typeof setting === 'boolean')
                return;
            const [settingCasingName] = setting;
            if (typeof settingCasingName === 'string' &&
                utils_1.isValidCaseName(settingCasingName)) {
                const validPropCases = utils_1.validCases[settingCasingName];
                if (utils_1.isRef(node))
                    return;
                if ('properties' in node && node.properties) {
                    Object.keys(node.properties).forEach(propName => {
                        const propCase = case_1.default.of(propName);
                        if (!validPropCases.has(propCase)) {
                            const correctVersion = settingCasingName in utils_1.validCases
                                ? case_1.default[settingCasingName](propName)
                                : '';
                            report(`Property "${propName}" has wrong casing.${correctVersion
                                ? ` Should be "${correctVersion}".`
                                : ''}`, [...location, 'properties']);
                        }
                    });
                }
                return;
            }
        },
    },
    isValidSetting: option => Array.isArray(option) && !!option[0] && utils_1.isValidCaseName(option[0]),
};
exports.default = rule;
