"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const case_1 = __importDefault(require("case"));
const utils_1 = require("../../utils");
const name = 'parameter-casing';
const rule = {
    name,
    visitor: {
        ParameterObject: ({ node, report, setting }) => {
            if (typeof setting === 'boolean')
                return;
            const [settingCasingName] = setting;
            if (typeof settingCasingName === 'string' &&
                utils_1.isValidCaseName(settingCasingName)) {
                const validParamCase = utils_1.validCases[settingCasingName];
                const nodeCase = case_1.default.of(node.name);
                if (!validParamCase.has(nodeCase)) {
                    const correctVersion = settingCasingName in utils_1.validCases
                        ? case_1.default[settingCasingName](node.name)
                        : '';
                    report(`Parameter "${node.name}" has wrong casing.${correctVersion
                        ? ` Should be "${correctVersion}".`
                        : ''}`);
                }
            }
        },
    },
    isValidSetting: option => Array.isArray(option) && !!option[0] && utils_1.isValidCaseName(option[0]),
};
exports.default = rule;
